@@ -2,6 +2,7 @@ import json
 import jwt
 import time
 import uuid
+import copy
 from py4web import request, Session
 from pydal.validators import Validator
 from yatl.helpers import (
@@ -84,7 +85,7 @@ class FormStyleFactory:
             input_id = "%s_%s" % (field.tablename, field.name)
             value = vars.get(field.name, field.default)
             error = errors.get(field.name)
-            field_class = field.type.split()[0].replace(":", "-")
+            field_class = "type-" + field.type.split()[0].replace(":", "-")
             placeholder = (
                 field._placeholder if "_placeholder" in field.__dict__ else None
             )
@@ -92,9 +93,14 @@ class FormStyleFactory:
             # only diplay field if readable or writable
             if not field.readable and not field.writable:
                 continue
-            # if this is a readonly form and the field is not readable, than do not show
-            if readonly and not field.readable:
-                continue
+            # if this is a reaonly field only show readable fields
+            if readonly:
+                if not field.readable:
+                    continue
+            # if this is an create form (unkown id) then only show writable fields
+            elif not vars.get('id'):
+                if not field.writable:
+                    continue
             # ignore blob fields
             if field.type == "blob":  # never display blobs (mistake?)
                 continue
@@ -156,10 +162,14 @@ class FormStyleFactory:
                     _title=title,
                 )
             elif field.type == "upload":
-                control = DIV(INPUT(_type="file", _id=input_id, _name=field.name))
+                control = DIV()
                 if value:
-                    control.append(A("download", _href=field.download_url(value)))
-                    control.append(
+                    download_div = DIV()
+                    download_div.append(LABEL("Currently:  ",))
+                    download_div.append(
+                        A(" download ", _href=field.download_url(value))
+                    )
+                    download_div.append(
                         INPUT(
                             _type="checkbox",
                             _value="ON",
@@ -167,7 +177,10 @@ class FormStyleFactory:
                             _title=title,
                         )
                     )
-                    control.append("(check to remove)")
+                    download_div.append(" (check to remove)")
+                    control.append(download_div)
+                control.append(LABEL("Change: "))
+                control.append(INPUT(_type="file", _id=input_id, _name=field.name))
             elif get_options(field.requires) is not None:
                 multiple = field.type.startswith("list:")
                 value = list(map(str, value if isinstance(value, list) else [value]))
@@ -184,6 +197,8 @@ class FormStyleFactory:
                 )
             else:
                 field_type = "password" if field.type == "password" else "text"
+                if field.type.startswith("list:"):
+                    value = json.dumps(value or [])
                 control = INPUT(
                     _type=field_type,
                     _id=input_id,
@@ -197,8 +212,9 @@ class FormStyleFactory:
             key = control.name.rstrip("/")
             if key == "input":
                 key += "[type=%s]" % (control["_type"] or "text")
-            control["_class"] = self.classes.get(key, "")
-
+            control["_class"] = (
+                control.attributes.get("_class", "") + " " + self.classes.get(key, "")
+            ).strip()
             controls["labels"][field.name] = field.label
             controls["widgets"][field.name] = control
             controls["comments"][field.name] = field.comment if field.comment else ""
@@ -207,12 +223,16 @@ class FormStyleFactory:
             if error:
                 controls["errors"][field.name] = error
 
-            if field.type == 'boolean':
+            if field.type == "boolean":
                 form.append(
                     DIV(
                         SPAN(control, _class=class_inner),
-                        LABEL(' ' + field.label, _for=input_id, _class=class_label,
-                              _style="display: inline !important"),
+                        LABEL(
+                            " " + field.label,
+                            _for=input_id,
+                            _class=class_label,
+                            _style="display: inline !important",
+                        ),
                         P(error, _class=class_error) if error else "",
                         P(field.comment or "", _class=class_info),
                         _class=class_outer,
@@ -228,8 +248,8 @@ class FormStyleFactory:
                         _class=class_outer,
                     )
                 )
-            
-            if "id" in vars:
+
+            if vars.get('id'):
                 form.append(INPUT(_name="id", _value=vars["id"], _hidden=True))
         if deletable:
             controls["delete"] = INPUT(
@@ -240,8 +260,16 @@ class FormStyleFactory:
             )
             form.append(
                 DIV(
-                    SPAN(controls["delete"], _class=class_inner, _stye="vertical-align: middle;"),
-                    P(" check to delete", _class="help", _style="display: inline !important"),
+                    SPAN(
+                        controls["delete"],
+                        _class=class_inner,
+                        _stye="vertical-align: middle;",
+                    ),
+                    P(
+                        " check to delete",
+                        _class="help",
+                        _style="display: inline !important",
+                    ),
                     _class=class_outer,
                 )
             )
@@ -280,6 +308,30 @@ def FormStyleBulma(table, vars, errors, readonly, deletable):
     return FormStyleDefault(table, vars, errors, readonly, deletable, classes)
 
 
+def FormStyleBootstrap4(table, vars, errors, readonly, deletable):
+    classes = {
+        "outer": "form-group",
+        "inner": "",
+        "label": "h4",
+        "info": "form-text",
+        "error": "form-text text-danger py4web-validation-error",
+        "submit": "btn btn-outline-info",
+        "input": "form-control",
+        "input[type=text]": "form-control",
+        "input[type=date]": "form-control",
+        "input[type=time]": "form-control",
+        "input[type=datetime-local]": "form-control",
+        "input[type=radio]": "form-check-input",
+        "input[type=checkbox]": "form-check-input",
+        "input[type=submit]": "btn btn-outline-info",
+        "input[type=password]": "form-control",
+        "input[type=file]": "form-control-file",
+        "select": "form-control",
+        "textarea": "form-control",
+    }
+    return FormStyleDefault(table, vars, errors, readonly, deletable, classes)
+
+
 # ################################################################
 # Form object (replaced SQLFORM)
 # ################################################################
@@ -310,7 +362,7 @@ class Form(object):
     :param signing_info: information that should not change between when the CSRF token is signed and
         verified.  This information is not leaked to the form.  For instance, if you wish to verify
         that the identity of the logged in user has not changed, you can do as below.
-        signing_info = session.get("user", {}).get("id", "")
+        signing_info = session.get('user', {}).get('id', '')
         The content of the field should be convertible to a string via json.
     """
 
@@ -368,7 +420,8 @@ class Form(object):
             if self.record:
                 self.vars = self._read_vars_from_record(table)
         else:
-            post_vars = self.vars = request.forms
+            post_vars = request.forms
+            self.vars = copy.deepcopy(request.forms)
             self.submitted = True
             process = False
 
@@ -384,8 +437,13 @@ class Form(object):
                     for field in self.table:
                         if field.writable and field.readable and field.type != "id":
                             original_value = post_vars.getall(field.name)
-                            if isinstance(original_value, list) and len(original_value) == 1:
+                            if (
+                                isinstance(original_value, list)
+                                and len(original_value) == 1
+                            ):
                                 original_value = original_value[0]
+                            if field.type.startswith("list:"):
+                                original_value = json.loads(original_value or "[]")
                             (value, error) = field.validate(original_value, record_id)
                             if field.type == "password" and record_id and value is None:
                                 continue
@@ -403,6 +461,7 @@ class Form(object):
                             validated_vars[field.name] = value
                             if error:
                                 self.errors[field.name] = error
+                    self.vars.update(validated_vars)
                     if validation:
                         validation(self)
                     if self.record and dbio:
